library(spider)
library(phangorn)

######################
#Read in data
ireCoiM <- read.dna("coi.fasta", format = "fasta")
ireCoiBM <- read.dna("coib.fasta", format = "fasta")
ire28SM <- read.dna("28s.fasta", format = "fasta")
ireCadM <- read.dna("cad.fasta", format = "fasta")
ireArgk <- read.dna("argk.fasta", format = "fasta")

######################
#Distance matrices
coiDist <- dist.dna(ireCoiM, model = "raw")
coiBDist <- dist.dna(ireCoiBM, model = "raw")
s28Dist <- dist.dna(ire28SM, model = "raw")
argkDist <- dist.dna(ireArgk, model = "raw")
cadDist <- dist.dna(ireCadM, model = "raw")
cadHDist <- dist.hamming(as.phyDat(ireCadM)) #Hamming distance to take into account polymorphisms

######################
#Species vectors
vecTrans <- data.frame(tagName = c("Ireanip", "Irecrini", "Iredupl", "Irepari", "Irestic", "Iremini"), pName = c("I. aniptus", "I. crinitus", "I. duplex", "I. parilis", "I. stichus", "I. minimus"))

coiVec <- sapply(strsplit(dimnames(ireCoiM)[[1]], split = "\\|"), function(xx) xx[3])
coiVec <- vecTrans$pName[match(coiVec, vecTrans$tagName)]

coiBVec <- sapply(strsplit(dimnames(ireCoiBM)[[1]], split = "\\|"), function(xx) xx[3])
coiBVec <- vecTrans$pName[match(coiBVec, vecTrans$tagName)]

s28Vec <- sapply(strsplit(dimnames(ire28SM)[[1]], split = "\\|"), function(xx) xx[3])
s28Vec <- vecTrans$pName[match(s28Vec, vecTrans$tagName)]

argkVec <- sapply(strsplit(dimnames(ireArgk)[[1]], split = "\\|"), function(xx) xx[3])
argkVec <- vecTrans$pName[match(argkVec, vecTrans$tagName)]

cadVec <- sapply(strsplit(dimnames(ireCadM)[[1]], split = "\\|"), function(xx) xx[3])
cadVec <- vecTrans$pName[match(cadVec, vecTrans$tagName)]

######################
#'Barcoding gap' plots
bgplot <- function(distobj, sppVec, plotLabel = "", cexPoints = 1.3, cexAxis = 1, cexLabel = 1, cexPlot = 1){
	inter <- nonConDist(distobj, sppVec) *100
	intra <- maxInDist(distobj, sppVec) *100
	min <- minInDist(distobj, sppVec) *100
	length(which(inter-intra <= 0))

	bnd <- cbind(data.frame(inter, intra, min, sppVec))
	#ord <- bnd
	ord <- bnd[order(bnd$sppVec),]

	plot(ord$inter, type="n", ylab="Uncorrected genetic distance (%)", xlab=NA, xaxt = "n", ylim = c(0, 12.5), cex.lab = cexAxis, cex.axis = cexAxis)

	segCol <- rep("grey60", length(ord$inter))
	segCol[ord$inter-ord$intra < 0] <- "red"
	segLine <- rep("15", length(ord$inter))
	segLine[ord$inter-ord$intra < 0] <- "solid"

	sppBGr <- cumsum(table(as.vector(sppVec))) + 0.5
	sppBGl <- c(0, sppBGr[-length(sppBGr)])
	rect(sppBGl, rep(-1, length(sppBGl)), sppBGr, rep(13, length(sppBGl)), col = c("white", "grey97"), border = NA)
	
	points(ord$min, pch = 8, col = "grey70", cex = cexPoints)
	points(ord$intra, pch = 15, col = "grey50", cex = cexPoints)
	points(ord$inter, pch = 19, col = "grey30", cex = cexPoints)
	segments(x0=1:length(ord$inter), y0=ord$inter, y1=ord$intra, col=segCol, lty = segLine, lwd=2)
	legend("topleft", legend = plotLabel, bty = "n", cex = cexLabel)
	axis(1, at = 1:length(ord$sppVec), labels = ord$sppVec, las = 2, cex.axis = cexAxis)
	invisible(ord)
}

#'Barcoding gap' plot as individual PDF figures
pdf(file = "/home/sam/Documents/PhD/aaaThesis/01taxonomy/images/IreBG_COI.pdf")
bgplot(coiDist, coiVec, plotLabel = "COI")
legend("topright", legend = c("Min. interspecific dist.", "Max. intraspecific dist.", "Min. intraspecific dist."), pch = c(19, 15, 8), col = c("grey30", "grey50", "grey70"), bty = "n")
dev.off()

pdf(file = "/home/sam/Documents/PhD/aaaThesis/01taxonomy/images/IreBG_COIB.pdf")
bgplot(coiBDist, coiBVec, plotLabel = "COIB")
dev.off()

pdf(file = "/home/sam/Documents/PhD/aaaThesis/01taxonomy/images/IreBG_28S.pdf")
bgplot(s28Dist, s28Vec, plotLabel = "28S")
dev.off()

pdf(file = "/home/sam/Documents/PhD/aaaThesis/01taxonomy/images/IreBG_ArgK.pdf")
bgplot(argkDist, argkVec, plotLabel = "ArgK")
dev.off()

pdf(file = "/home/sam/Documents/PhD/aaaThesis/01taxonomy/images/IreBG_CAD.pdf")
bgplot(cadDist, cadVec, plotLabel = "CAD")
dev.off()

pdf(file = "/home/sam/Documents/PhD/aaaThesis/01taxonomy/images/IreBG_CADH.pdf")
bgplot(cadHDist, cadVec, plotLabel = "CAD Hamming")
dev.off()

tlmtext <- function(text, offset, ctext = 1.5){
	pl <- par("usr")
	par(xpd = TRUE)
	text(pl[1] - offset * (pl[2] - pl[1]), pl[4] + offset * (pl[4] - pl[3]), text, cex=ctext)
	par(xpd = FALSE) 
}

#'Barcoding gap' plots as a single EPS file, as published
setEPS()
postscript(file = "/home/sam/Documents/My_Papers/Irenimus/03_publication/figures_87-91.eps", paper = "special", height = 5.58, width = 6.69)
layout(matrix(c(rep(c(rep(1,3), rep(2,3)), 3), rep(c(rep(3,2), rep(4,2), rep(5,2)), 2)), ncol = 6, byrow = TRUE))
bgplot(coiBDist, coiBVec, plotLabel = "COIB")
tlmtext("88", 0.09)
bgplot(coiDist, coiVec, plotLabel = "COI",  cexPoints = 1.3, cexAxis = 1, cexLabel = 1)
tlmtext("87", 0.09)
legend("topright", legend = c("Min. interspecific dist.", "Max. intraspecific dist.", "Min. intraspecific dist."), pch = c(19, 15, 8), col = c("grey30", "grey50", "grey70"), bty = "n", cex = 0.6)
bgplot(argkDist, argkVec, plotLabel = "ArgK",  cexPoints = 0.8, cexAxis = 0.5, cexLabel = 0.9, cexPlot = 0.5)
tlmtext("90", 0.09)
bgplot(cadDist, cadVec, plotLabel = "CAD",  cexPoints = 0.8, cexAxis = 0.5, cexLabel = 0.9, cexPlot = 0.5)
tlmtext("91", 0.09)
bgplot(s28Dist, s28Vec, plotLabel = "28S",  cexPoints = 0.8, cexAxis = 0.5, cexLabel = 0.9, cexPlot = 0.5)
tlmtext("89", 0.09)
dev.off()

######################
#Principal coordinates analysis of distance data
#Not published
coiOrd <- ordinDNA(coiDist, coiVec)
s28Ord <- ordinDNA(s28Dist, s28Vec)
argkOrd <- ordinDNA(argkDist, argkVec)
cadOrd <- ordinDNA(cadDist, cadVec)
cadHOrd <- ordinDNA(cadHDist, cadVec)


######################
#Nucleotide diagnostic plots
ndplot <- function(DNAbin, sppVec, cexText = 1){
	DNAbin <- as.matrix(DNAbin)
	nd <- nucDiag(DNAbin, sppVec)
	nd <- nd[sort(names(nd))]
	lnd <- length(nd)
	end <- dim(DNAbin)[2]
	
	op <- par(mar = c(0, 0, 0, 0) + 0.5)
	plot(1, 1, type = "n", ylim = c(0, (lnd*3) + 2), xlim = c(0, end + 160), bty = "n", xaxt = "n", yaxt = "n", xlab = NA, ylab = NA)
	rect(rep(1, lnd), (1:lnd) * 3, rep(end, lnd), (1:lnd) * 3 + 1, col = "grey90", border = "grey90")
	text(rep(end + 1, lnd), (1:lnd) * 3 + 0.5, labels = paste(names(nd), " (", table(as.vector(sppVec)), ")", sep = ""), pos = 4, cex = cexText)
	
	par(op)
	
	top <- 4
	bottom <- 3
	out <- list()
	length(out) <- lnd
	
	slashcoll <- function(vector) paste(unique(vector), collapse = "|")
	baseCol <- data.frame(base = c("a", "c", "g", "t", "t|c", "a|c", "g|a", "a|m", "a|r", "g|r", "t|y"), col =c("#69FFB5", "#69B3FF", "#FFB569", "#FF69B3", rep("grey60", 7)), stringsAsFactors = FALSE)
	#baseCol <- data.frame(base = c("a", "c", "g", "t", "t|c", "a|c", "g|a", "a|m", "a|r", "g|r", "t|y"), col =c("#008000", "blue", "black", "red", rep("grey50", 7)), stringsAsFactors = FALSE)
	
	for(i in 1:lnd){
		if(length(nd[[i]]) > 0){
		ndcm <- as.character(DNAbin)[which(sppVec == names(nd)[i]) , nd[[i]]]
		if(class(ndcm) == "matrix") ndc <- unlist(apply(ndcm, 2, slashcoll)) else ndc <- ndcm
		#print(ndc)
		sites <- nd[[i]]
		ndcc <- baseCol$col[match(ndc, baseCol$base)]
		segments(sites, rep(bottom, length(sites)), sites, rep(top, length(sites)), col = ndcc, lwd = 2, lmitre = 1)
		text(sites, rep(top + 0.25, length(sites)) + rep_len(c(0.09, 0.00, -0.09), length(sites)), labels = sites, cex = 0.4 * cexText)
		text(sites, rep(bottom - 0.25, length(sites)), labels = ndc, cex = 0.4 * cexText)
		} else {
		text(end/2, bottom + 0.5, labels = "No diagnostic nucleotides", cex = cexText)
		}
		top <- top + 3
		bottom <- bottom + 3
		
	}
	
	return(nd)
}

#Nucleotide diagnostics plots as individual PDF files
pdf(file = "/home/sam/Documents/PhD/aaaThesis/01taxonomy/images/IreND_COI.pdf")
coiDiag <- ndplot(ireCoiM, coiVec)
legend("topleft", legend = "COI", bty = "n")
dev.off()

pdf(file = "/home/sam/Documents/PhD/aaaThesis/01taxonomy/images/IreND_COIB.pdf")
coiBDiag <- ndplot(ireCoiBM, coiBVec)
legend("topleft", legend = "COI", bty = "n")
dev.off()

pdf(file = "/home/sam/Documents/PhD/aaaThesis/01taxonomy/images/IreND_28S.pdf")
s28Diag <- ndplot(ire28SM, s28Vec)
legend("topleft", legend = "28S", bty = "n")
dev.off()

pdf(file = "/home/sam/Documents/PhD/aaaThesis/01taxonomy/images/IreND_ArgK.pdf")
argkDiag <- ndplot(ireArgk, argkVec)
legend("topleft", legend = "ArgK", bty = "n")
dev.off()

pdf(file = "/home/sam/Documents/PhD/aaaThesis/01taxonomy/images/IreND_CAD.pdf")
cadDiag <- ndplot(ireCadM, cadVec)
legend("topleft", legend = "CAD", bty = "n")
dev.off()


#Nucleotide diagnostic plot as single EPS file, as published
trmtext <- function(text, offset, ctext = 1.5){
	pl <- par("usr")
	par(xpd = TRUE)
	text(pl[2] - offset * (pl[2] - pl[1]), pl[4] - 0.25* offset * (pl[4] - pl[3]), text, cex=ctext)
	par(xpd = FALSE) 
}

setEPS()
postscript(file = "/home/sam/Documents/My_Papers/Irenimus/03_publication/figures_92-96.eps", paper = "special", height = 5.58, width = 6.69)
layout(matrix(c(rep(c(rep(1,3), rep(2,3)), 3), rep(c(rep(3,2), rep(4,2), rep(5,2)), 2)), ncol = 6, byrow = TRUE))

coiBDiag <- ndplot(ireCoiBM, coiBVec, cexText = 0.9)
trmtext("COIB", 0.16, ctext = 1)
tlmtext("93", -0.02)

coiDiag <- ndplot(ireCoiM, coiVec, cexText = 0.9)
trmtext("COI", 0.16, ctext = 1)
tlmtext("92", -0.02)

cadDiag <- ndplot(ireCadM, cadVec, cexText = 0.5)
trmtext("CAD", 0.16, ctext = 1)
tlmtext("96", -0.02)

argkDiag <- ndplot(ireArgk, argkVec, cexText = 0.5)
trmtext("ArgK", 0.16, ctext = 1)
tlmtext("95", -0.02)

s28Diag <- ndplot(ire28SM, s28Vec, cexText = 0.5)
trmtext("28S", 0.16, ctext = 1)
tlmtext("94", -0.02)
dev.off()


######################
#Summary statistics of diagnostic nucleotides
summary(sapply(s28Diag, length))
summary(sapply(coiDiag, length))
summary(sapply(argkDiag, length))
summary(sapply(cadDiag, length))

######################
#Nucleotide diagnostics of sequences randomly assigned to "taxa"
 rnucDiag(ireCoiM, coiVec)
 rnucDiag(ire28SM, s28Vec)
 rnucDiag(ireArgk, argkVec)
 rnucDiag(ireCadM, cadVec)
